"""__init__.py: main PyMidas  package initialization
 
Checks sys.argv[0] == 'midas' to determine whether Midas initialization
is done verbosely or quietly.
 
 
"""
##############################################################################
#
#   This file is part of PyMidas .
#
#   PyMidas is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   PyMidas is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with PyMidas; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
##############################################################################
#
#  Author:             S. Maisala       
#  Email:              pymidas@eso.org
#  Contact:            www.eso.org/sampo/pymidas
#
##############################################################################
#
#   Change History log
#
# Num  Date      Revision  Who   What
#
# 00   2005/07/15  0.95    SM   Creation of this History  
# 
# 01   2005/11/03  1.0.0   SM  Version 1.0.0
#
# 02   2005/11/14  1.0.1   SM  Corrected typo import wutil
#
# 03   2005/11/17  1.0.2   SM  Corrected C-layer version
#
# 04   2005/11/21  1.0.3   SM  Initialization text information updated.
#                              Setup.py compile options changed.
###############################################################################



__version__ = "1.0.5 (2006 March 6)"

global variable
import sys, os, string, __main__ , _inmidas


def usage():
    print __main__.__doc__
    sys.stdout.flush()
    sys.exit()


# follow links to get to the real executable filename
executable = sys.argv[0]
while os.path.islink(executable):
    executable = os.readlink(executable)


_midasMain = os.path.split(executable)[1] != 'pymidas'

del executable

import getopt
try:
	optlist, args = getopt.getopt(sys.argv[1:],"hdsfm",
            ["commandwrapper=", "help", "debug","socket","files", "mimic"])
        if len(args) > 1:
            print 'Error: more than one savefile argument'         
except getopt.error, e:
        print str(e)




# Check LD_LIBRARY PATH. This must be set before launching pymidas!
#ld_lib_path=''
#if os.environ.has_key( 'LD_LIBRARY_PATH' ):
#    ld_lib_path=os.environ[ 'LD_LIBRARY_PATH' ]

#if( 'midas' in ld_lib_path ):
#    newpath = ld_lib_path.split( ':' )
#    for i in newpath:
#        if 'midas' in i:
#            midaslibpath = i
#            #print 'MIDAS LD_LIBRARY_PATH:', midaslibpath
#else:
#	if optlist:
#       	for opt, value in optlist:
#	 		if opt in ("-h", "--help"):
#                		print __main__.__doc__
#				sys.stdout.flush()
#
#
#
#    	raise SystemExit("""
#    	Your midas LD_LIBRARY_PATH environment variable is not defined.
#    	Before starting PyMidas, define it by doing (for example)
#    	setenv LD_LIBRARY_PATH /usr/local/midas/05JUN/lib
#    	at the Unix command line. The values will depend on your MIDAS
#    	installation.
#    """)




# Setting Midas version and location
if os.environ.has_key( 'MIDASHOME' ):
	os.environ[ 'MIDASHOME0' ]=  os.environ[ 'MIDASHOME' ]


else:
	raise SystemExit("""
	Your midas MIDASHOME environment variable is not defined.
	Before starting PyMidas, define it by doing (for example)
	setenv MIDASHOME /usr/local/midas
	at the Unix command line. The values will depend on your MIDAS
	installation.
    """)


if os.environ.has_key( 'MIDVERS' ):
        os.environ[ 'MIDVERS0' ]=  os.environ[ 'MIDVERS' ]

else:
	raise SystemExit("""
 	Your midas MIDVERS environment variable is not defined.
	Before starting PyMidas, define it by doing (for example)
	setenv MIDVERS 05SEP
	at the Unix command line. The values will depend on your MIDAS
	installation.
    """)



#print 'MIDAS home:',os.environ[ 'MIDASHOME0' ]
#print 'MIDAS version:', os.environ[ 'MIDVERS0' ]
#os.environ[ 'MIDVERS' ]   = os.environ[ 'MIDVERS0' ]
#os.environ[ 'MIDASHOME' ] = os.environ[ 'MIDASHOME0' ]
#os.environ[ 'MID_WORK' ]  = os.environ[ 'HOME' ]+ '/midwork/'


#print
#print "Pymidas initialization version ",__version__
#print





if _midasMain:
    # if not executing as pymidas main, just initialize midas module
    # quietly load initial MIDAS symbols and packages
    # 'Still under development...'

    
    # Set Midas envinroment variables
    _inmidas.environment( 0 ) # 0 = no debugging

                
    import _pymidasInit
   
    # load initial midas symbols and packages
    # 2 is special case for importing outside of pymidas
    _pymidasInit.PymidasInitialize( 2 )

    
else:
    # special initialization when this is the main program

    # command-line options

    verbose = 0
    doCmdline = 1
    _debug = 0
    _socket = 1
    _mimic = 0
    if optlist:
        for opt, value in optlist:
            if opt == "-i":
                doCmdline = 0
            elif opt == "--commandwrapper":
                if value in ("yes", "y"):
                    doCmdline = 1
                elif value in ("no", "n"):
                   doCmdline = 0
                else:
                    usage()
            elif opt in ("-v", "--verbose"):
                verbose = verbose + 1
            elif opt in ("-h", "--help"):
                usage()
            elif opt in ("-d", "--debug"):
                _debug = 1
            elif opt in("-s", "--socket"):
                _socket = 1
            elif opt in("-f", "--files"):
                _socket = 0
            elif opt in("-m", "--mimic"):
                _mimic = 1
                
            else:
                print "Program bug, uninterpreted option", opt
                raise SystemExit
                
    del getopt, verbose, usage, optlist


  


    # Set Midas envinroment variables
    _inmidas.environment( _debug )

    import _pymidasInit
   

    # load initial midas symbols and packages
    if args:
        _pymidasInit.PymidasInitialize( _debug, _socket, _mimic )
    else:
        _pymidasInit.PymidasInitialize( _debug, _socket, _mimic)
    del args

    




# Initialize Pymidas session

# status =_pymidasInitCom.PymidasInitialize()



sys.stdout.flush()




