"""module _pymidasInitCom.py -- defines PymidasInitialize 

This module defines methods to be used in PyMidas intialization.

"""
##############################################################################
#   This file is part of PyMidas.
#
#   PyMidas is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   PyMidas is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with PyMidas; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
##############################################################################
#
#  Author:             S. Maisala       
#  Email:              pymidas@eso.org
#  Contact:            www.eso.org/sampo/pymidas
#
###############################################################################
#
#   Change History log
#
# Num  Date      Revision  Who   What
#
# 00   2005/09/19  0.95    SM   Creation of this History  
#
#
# 01   2005/09/19  0.95    SM   SOMO-6GHFKQ
#                               New session parameter asking method
#                               improved
#
# 02   2005/11/03  1.0.0   SM  Version 1.0.0
###############################################################################



import os, sys, string, midas,_pymidas_c_00
import  midastask, pymidasCom

_pymidas = _pymidas_c_00

del _pymidas_c_00

class PymidasInitialize( midas.PymidasInit ):

    """Pymidas initialize class"""


    def __init__( self, _debug = 0, _socket = 1, _mimic = 0 ):
        """ Midas session initialization"""
        if _debug == 2: # We are importing from pymidas import midas
            pymidasCom.globals.variable = 1
            pymidasCom.globals.imported = 1
            self.midasText()
            _debug = 0
        if _debug ==1:
            print 'debug =',_debug
            print 'connection type( files = 0, socket = 1):',_socket
        status, myUnit, cid = self.setSession(_socket, _debug)

        if status:
            raise PyMidasError('Cannot set PyMidas Session)')                           
        status = self.initPymidasLayer(_debug,_socket, myUnit,myUnit,cid )
        if status:
            raise PyMidasError('Cannot initialize Pymidas C-layer)')
        pymidasCom.globals.running = 1
        # set MIDAS globals
        pymidasCom.globals.running = 1
        pymidasCom.globals.mimic = _mimic
        pymidasCom.globals.socket = _socket
        pymidasCom.globals.debug = _debug
        comms,progs = pymidasCom.readCommands()
        self.initTasks()
        if _debug:
            self.midasText()
        

    def initPymidasLayer(self, _debug, _socket, remotUnit, myUnit, cid  ):
        """ Initialize pymidas C- layer"""
        # Set debugging level #
        _pymidas.SetParam ('TRACE_LEVEL',str( _debug ) )
        _pymidas.SetParam ('TRACE_CONSOLE', str( _debug ) )
        _pymidas.SetParam ('TRACE_FILE','1')
        # Set communication via files or sockets
        _pymidas.SetParam ('COMM_MODE',str( _socket) ) 
        # Set output via files or files+terminal (default files)
        _pymidas.SetParam ('COMM_OUT','1')
        
        status = _pymidas.Initialize( myUnit, myUnit, cid )
        
        return status



    def setSession( self, _socket, _debug):
        """Set MIDAS session up"""          

        #os.environ['MIDAS_OUTSIDE']='YES'
        os.environ[ 'DAZUNIT' ]      =''


        # if necessary create MIDAS work directory

        if os.environ.has_key( 'MID_WORK' ):
            if _debug ==1:
                print 'MIDAS working directory', os.environ[ 'MID_WORK' ]
            path = os.environ[ 'MID_WORK' ]
            if path[-1] <> '/':
                path=path+'/'
        else:
            if os.path.exists( os.path.join( os.environ[ 'HOME' ],'midwork' )):
                path = os.path.join( os.environ[ 'HOME' ], 'midwork' )
                if _debug ==1:
                    print 'MIDAS default working directory:',path
                path  = path+'/'
            else:
                if _debug ==1:
                    print 'MIDAS working directory does not exist!'
                path=os.path.join( os.environ[ 'HOME' ],'midwork/' )
                os.mkdir(path)
                if _debug ==1:
                    print 'Creating default working directory:', path

                os.environ[ 'MID_WORK' ]  = path



        
        # Set default DAZUNIT to the first free unit
        status = -1
        while status != 0:
            status, cid, dazUnit, session, parallel = self.setDazUnit( _debug )
        if _debug:    
            print 'cid number:', cid
            print 'DAZUNIT:', dazUnit
            print 'Parallel Session:',parallel
        os.environ['PARALLEL'] = parallel

        keyname = 'FORGR' + dazUnit + '.KEY'
        path = os.environ[ 'MID_WORK' ] 

        # clean old sessions
        self.cleanSession( _debug )        
        self.newSession( _debug )

        # Here for X Workstations
        
        os.environ[ 'WORK_STATION' ] = 'DAZ'+ os.environ[ 'DAZUNIT' ]
        os.environ[ 'I_TYPE' ] = 'SXW  '
        os.environ[ 'AGL3DEV' ] = 'xterm'
        #print 'session', session

        if session == 1:
            #Create Python session pid file
            if _debug:
                print "Creating PYMIDAS" + dazUnit+'.PID'
            filename= 'PYMIDAS' + dazUnit + '.PID'
            pyPid = os.getpid( )
            file = open(os.path.join(  path, filename ), 'w' )
            file.write( str( pyPid ) )
            file.close() 
   
            # Open background midas session

            
            # Check if midas login.prg exists
            filename =  os.path.join(path,'login.prg')
            if  os.path.exists( filename ):
                filename1 =  os.path.join(path,'login_old.prg')
                if _debug:
                    print 'file', os.path.join(path,'login.prg'), 'found.'
                    # rename it to login_old.prg
                os.rename( filename, filename1 )
                
            # Create new login file                   
            file = open(os.path.join(  path, 'login.prg' ), 'w' ) 
            #print os.path.join(  path, 'login.prg' )
            #file.write( "SET/MIDAS OUTPUT=YES" )

            # setting midas in bg mode via files or sockets
            if _socket:
                 file.write( "SET/BACK SOCKETS OFF \n " )
            else:
                 file.write( "SET/BACKROUND FILES\n " )
            
            if  os.path.exists( os.path.join(path,'login_old.prg')):
                loginfile = open(os.path.join(  path, 'login_old.prg' ), 'r' )
                line ='notNULL'
                while line !='':
                    line = loginfile.read()
                    #print line
                    file.write( line ) 
                loginfile.close()

            file.close()                            # Close file
    
            # Execute background MIDAS
            print "Setting up Midas session."
                        #tbmod.append(tb1)
            #exec_com = os.environ[ 'MID_MONIT' ]+'/prepa.exe >/dev/null & '
            exec_com = os.environ[ 'MID_MONIT' ]+'/prepa.exe &'
            status = os.system( exec_com  )
            if _debug:
                print 'Setting Background midas session status ',status
    
            # Check if background midas is running

            status = self.waitBkgMidas( dazUnit ,_debug)


            # If we have a original login.prg let's put it back
            filename =  os.path.join( path, 'login_old.prg' )
            filename1 = os.path.join( path, 'login.prg' )
            if  os.path.exists( filename):
                if _debug:
                    print 'renaming', filename,'-->', filename1
                os.rename(filename,filename1)
            elif os.path.exists(filename1):
                os.remove( filename1 )
            # initialize pymidas.so layer
        return status, dazUnit, cid




    def initTasks(self):
        """Init midas tasks"""        
        comms = pymidasCom.MidasCommands()
        for tasks in comms:
            tasks = tasks.lower( )
            tasks = pymidasCom.midas2py(tasks)
            setattr( midas, tasks, midastask.MidasTask(tasks) )
        # Add Pymidas Extra commands
        setattr( midas, 'do', pymidasCom._do ) 
        #setattr( midas, 'getKeyword' , pymidasCom._getKeyVal )
        setattr( midas, 'putKeyword' , pymidasCom._putKeyVal )
        setattr( midas, 'getPix', pymidasCom._getPix )
        setattr( midas, 'putPix' , pymidasCom._putPix )
        setattr( midas, 'getDesc' , pymidasCom._getDesc )
        setattr( midas, 'putDesc' , pymidasCom._putDesc )
        setattr( midas, 'cd', pymidasCom.chdir )
        setattr( midas, 'pwd', pymidasCom.pwd )



    def midasText(self,banner="\n\n"
        "          SAMPO-PyMIDAS on PC/Linux\n"
        "*****************************************************************************\n"
        "If this module is imported from python (>> from pymidas import midas)\n"      
        "Give keyword database query in format:\n\n"
        "Old format MIDAS syntax can be used  by running:\n"
        "result  = midas.do('a/b x,y,z')\n\n"
        "All other midas commands can be run by using python syntax:\n"
        "result = midas.aB(x,y,z)\n\n"):
        """Start interpreter"""
    
        print banner





    def readString( self, string ):
        """ Read string """
        string = str( raw_input( string ) )
        return string




    def cleanSession( self, _debug ):
        """ Clean midas session. Remove all session files and kill processes"""
        
        midwork =  os.environ[ 'MID_WORK' ]
        dazUnit=   os.environ[ 'DAZUNIT' ]

        # remove file RUNNINGXX          
        filename = os.path.join( midwork, 'RUNNING' + dazUnit)
        if os.path.exists(filename):
            os.remove( filename )

        # remove file sxwXX0.dat          
        filename = os.path.join( midwork, 'sxw'+ dazUnit + '0.dat')
        if os.path.exists(filename):
            os.remove( filename )

        # remove FORGRxx00.RBOX
        filename =  os.path.join( midwork, 'FORGR'+ dazUnit + '00.RBOX' )
        if os.path.exists(filename):
            os.remove( filename )

        # remove FORGRxx.SBOX
        filename =  os.path.join( midwork, 'FORGR'+ dazUnit + '.RBOX' )
        if os.path.exists(filename):
            os.remove( filename )

        # remove FORGRxx.SBOX
        filename =  os.path.join( midwork, 'FORGR'+ dazUnit + '.SBOX' )
        if os.path.exists(filename):
            os.remove( filename )     

        # remove FORGR__xx.RBOX
        filename =  os.path.join( midwork, 'FORGR__'+ dazUnit + '.RBOX' )
        if os.path.exists(filename):
            os.remove( filename )


        # remove xhelpxx=
        filename =  midwork + 'xhelp'+ dazUnit
        if os.path.exists(filename):
            os.remove( filename )


        # remove midas_xwxx
        filename =  midwork + 'midas_xw'+ dazUnit 
        if os.path.exists(filename):
            os.remove( filename )

        #Remove login.prg
        #filename =  os.path.join( midwork, 'login.prg' )
        #if os.path.exists(filename):
        #    os.remove( filename )


        #Remove getXdepth.env
        filename =  os.path.join( midwork, 'getXdepth.env' )
        if os.path.exists(filename):
            os.remove( filename ) 

        # remove midtemp.xx
        filename =  os.path.join( midwork, 'midtemp'+ dazUnit + '.prg' )
        if os.path.exists(filename):
            os.remove( filename )

         # remove midtemp.xx
        filename =  os.path.join( midwork, 'iditemp'+ dazUnit + '.dat' )
        if os.path.exists(filename):
            os.remove( filename )

        # remove iditemp
        filename =  os.path.join( midwork, 'iditemp'+ dazUnit + '0.dat' )
        if os.path.exists(filename):
            os.remove( filename )

        # remove FORGRXX.KEY
        #filename =  os.path.join( midwork, 'FORG'+ dazUnit + '.KEY' )
        #if os.path.exists(filename):
        #    os.remove( filename )


        # Find old session PID file
        filename = midwork + 'MIDAS' + dazUnit+ '.PID'
        pid = getPid( filename )
        if pid :    
            # Killing background midas session
            if _debug ==1:
                print 'Killing background Midas. pid ',pid
            killProcess( pid,_debug )
            # remove file 
            os.remove( filename )

        filename = midwork + 'idiserv' + dazUnit+ '.log'
        pid =  getPidIDI( filename )
        if pid :    
            # Killing background midas idiserver
            if _debug ==1:
                print 'Killing Midas idiserver. pid ',pid
            killProcess ( pid, _debug )
            # remove file 
            os.remove( filename )


        filename = midwork + 'PYMIDAS' + dazUnit+ '.PID'
        pid = getPid( filename )
        if pid :    
            # Killing background midas session
            #print 'Killing  PyMidas session  pid ',pid
            #killProcess( pid )
            # remove file 
            os.remove( filename )

        # remove Midas_osxxx
        filename =  os.path.join( midwork, 'Midas_osx'+ dazUnit )
        if os.path.exists(filename):
            os.remove( filename )




    def clearAllSessions( self, _debug ):
        """Clear all possible MIDAS sessions"""
        midwork =  os.environ[ 'MID_WORK' ]
        filelist=os.listdir( midwork )
        print 'Warning! Deleting all old sessions...'
        string = self.readString( 'Continue (y/n) ')
        if string[ 0 ] =='y' or string[ 0 ] == 'Y': 
            for filename in filelist:


                if 'RUNNING' in filename:
                    os.remove( os.path.join( midwork, filename ) )

                if 'FORG' in filename:
                    os.remove( os.path.join( midwork, filename ) )


                if 'midas_xw' in filename:
                    os.remove( os.path.join( midwork, filename ) )             

                if 'xhelp' in filename:
                    os.remove( os.path.join( midwork, filename ) )

                if 'midtemp' in filename:
                    os.remove( os.path.join( midwork, filename ) )

                if 'idisave' in filename:
                    os.remove( os.path.join( midwork, filename ) )             

                if 'getXdepth' in filename:
                    os.remove( os.path.join( midwork, filename ) )             

                if 'IDISERV' in filename:
                    os.remove( os.path.join( midwork, filename ) )

                if 'sxw' in filename:
                    os.remove( os.path.join( midwork, filename ) )


                if 'MIDAS' in filename:
                    pid = getPid( os.path.join( midwork, filename ))
                    if _debug:
                        print filename, 'pid', pid
                    if pid:
                        if _debug:
                            print 'pid found ',pid
                        killProcess( pid, _debug )
                    os.remove( os.path.join( midwork, filename ) )

                if 'idiserv' in filename:
                    pid = getPidIDI( os.path.join( midwork, filename ) )
                    if _debug:
                        print ' old idiserv  pid ', pid
                    if pid:
                        killProcess( pid, _debug )

                    os.remove( os.path.join( midwork, filename ) )


                if 'Midas_osx'  in filename:
                    os.remove( os.path.join( midwork, filename ) )
                    

        else:
            sys.exit( )



    def newSession( self,_debug ):
        """ New midas session keyfile generation. Run midas unix command: midasgo.exe"""
        
        # We must run midas initialization command 'midasgo.exe' to create
        # required KEYWORD database file FORGXX.KEY
        if _debug:
            print 'New session. Creating  MIDAS keyfile.'        
        exec_com = os.path.join( os.environ[ 'MID_MONIT' ], 'midasgo.exe' )
        status = os.system( exec_com )



    def setDazUnit( self,_debug ):
        """ Setting MIDAS environment variable DAZUNIT"""
        status = -1
        cid = 0
        session = 0
        midwork =  os.environ[ 'MID_WORK' ]
        filelist = os.listdir( midwork )
        foundunits = [ ]
        for k in filelist:
            if 'RUNNING' in k:
                if _debug:
                    print k
                
                units = k.replace( 'RUNNING', '' )
                foundunits.append( units )
                foundunits.sort( ) # sort the list
                if _debug:
                    print 'foundunits ', foundunits.__len__( )
        # print filelist
        if foundunits.__len__( )< 1: #No RUNNING sessions found.Setting the default
            dazUnit = "00"             
            status = 0
            session = 1
            parallel = 'noparallel'

        else:
            print 'WARNING: other MIDAS sessions might be running.'
            print 'Do you want to begin:'
            print '                     1.PARALLEL mode new session.' 
            print '                     2 New session.'
            print '                     3.Exit.'
            print ''
            print 'In New Session  all previous MIDAS sessions will be removed'

            # (1) SOMO-6GHFKQ
            status = 1
            while status: 
                string = self.readString( 'selection (1,2,3)> ')
                if string =='':
                    string ='1'
                if string in ['1', '2' ,'3']:
                    status = 0
            
            if int( string ) == 3:
                sys.exit( )

            elif int( string ) == 2:
                    self.clearAllSessions( _debug )
                    dazUnit = '00' # default 00
                    status = 0
                    cid = 0  # cid always 0 if only one session
                    session = 1
                    parallel = 'noparallel'
           
            
            else: # Default new parallel session
                print 'New parallel session. The following units are not available'
                parallel = 'parallel'
                for k in foundunits:
                    print k
                # BUG---
                status = 1
                while status:
                    string = str( raw_input( "Give session number>" ) )
                    if foundunits.__contains__( string ):
                        print 'Wrong session number:', string + ' Exiting'
                        sys.exit( )
                    elif (string.__len__( ) < 2 or string.__len__( ) > 2) :
                        print 'Unit number must be given in format XX :', string + \
                              ' is not valid'
                        status = 1
                    else:
                        dazUnit = string
                        status = 0
                        cid = foundunits.__len__( )  # cid number 
                        session = 1

        os.environ[ 'DAZUNIT' ] = dazUnit
        return status, cid , dazUnit, session, parallel

   



    def waitBkgMidas( self, dazUnit, _debug ):
        """ Wait background midas session getting up before continuing..."""
        import time
        midwork =  os.environ[ 'MID_WORK' ]
        filename = midwork + 'MIDAS' + dazUnit+ '.PID'

        status = 0
        i = 0
        while not status:
                status = os.path.exists( filename )
                if ( i > 5 ):
                    print 'Cannot find background Midas Session. Exiting...'
                    sys.exit()
                if status:

                    file = open( filename, 'r' )
                    try:
                        pid = int( file.readline( ) )
                        file.close( )
                    except ValueError:
                        print 'Warning! Cannot get PID number from file ', filename
                        return 

                    if _debug:
                        print 'Background Midas Session pid ',pid
                    file.close( )
                time.sleep( 0.5 )
                i = i+1
        status = 0
        i=0
        while not status:    
                status = pidStillRunning( pid )
                if ( i > 5 ):                
                    print 'Cannot find running  background Midas Session PID Exiting...'
                    sys.exit()
                if status:
                    if _debug:
                        print 'Background Midas session PID=',pid,'running ...'
                     #Remove login.prg file 
                     #filename =  os.path.join( midwork, 'login.prg' )
                     #if os.path.exists(filename):
                     #    os.remove( filename )

                    
                     
                         
                time.sleep( 0.5 )         
                i = i + 1     




# Public methods

def cleanSession( _debug ):
    """ Clean midas session. Remove all session files and kill processes"""

    midwork =  os.environ[ 'MID_WORK' ]
    dazUnit=   os.environ[ 'DAZUNIT' ]

    # remove file RUNNINGXX          
    filename = os.path.join( midwork, 'RUNNING' + dazUnit)
    if os.path.exists(filename):
        os.remove( filename )

    # remove file sxwXX0.dat          
    filename = os.path.join( midwork, 'sxw'+ dazUnit + '0.dat')
    if os.path.exists(filename):
        os.remove( filename )

    # remove FORGRxx00.RBOX
    filename =  os.path.join( midwork, 'FORGR'+ dazUnit + '00.RBOX' )
    if os.path.exists(filename):
        os.remove( filename )

    # remove FORGRxx.SBOX
    filename =  os.path.join( midwork, 'FORGR'+ dazUnit + '.RBOX' )
    if os.path.exists(filename):
        os.remove( filename )

    # remove FORGRxx.SBOX
    filename =  os.path.join( midwork, 'FORGR'+ dazUnit + '.SBOX' )
    if os.path.exists(filename):
        os.remove( filename )     

    # remove FORGR__xx.RBOX
    filename =  os.path.join( midwork, 'FORGR__'+ dazUnit + '.RBOX' )
    if os.path.exists(filename):
        os.remove( filename )


    # remove xhelpxx=
    filename =  midwork + 'xhelp'+ dazUnit
    if os.path.exists(filename):
        os.remove( filename )


    # remove midas_xwxx
    filename =  midwork + 'midas_xw'+ dazUnit 
    if os.path.exists(filename):
        os.remove( filename )

    #Remove login.prg
    #filename =  os.path.join( midwork, 'login.prg' )
    #if os.path.exists(filename):
    #    os.remove( filename )


    #Remove getXdepth.env
    filename =  os.path.join( midwork, 'getXdepth.env' )
    if os.path.exists(filename):
        os.remove( filename ) 

    # remove midtemp.xx
    filename =  os.path.join( midwork, 'midtemp'+ dazUnit + '.prg' )
    if os.path.exists(filename):
        os.remove( filename )

     # remove midtemp.xx
    filename =  os.path.join( midwork, 'iditemp'+ dazUnit + '.dat' )
    if os.path.exists(filename):
        os.remove( filename )

    # remove sxw.xx0
    filename =  os.path.join( midwork, 'iditemp'+ dazUnit + '0.dat' )
    if os.path.exists(filename):
        os.remove( filename )

    # remove FORGRXX.KEY
    #filename =  os.path.join( midwork, 'FORG'+ dazUnit + '.KEY' )
    #if os.path.exists(filename):
    #    os.remove( filename )


    # Find old session PID file
    filename = midwork + 'MIDAS' + dazUnit+ '.PID'
    pid = getPid( filename )
    if pid :    
        # Killing background midas session
        if _debug:
            print 'Killing background Midas. pid ',pid
        killProcess( pid, _debug )
        # remove file 
        os.remove( filename )

    filename = midwork + 'idiserv' + dazUnit+ '.log'
    pid =  getPidIDI( filename )
    if pid :    
        # Killing background midas idiserver
        if _debug:
            print 'Killing Midas idiserver. pid ',pid
        killProcess ( pid, _debug )
        # remove file 
        os.remove( filename )


    filename = midwork + 'PYMIDAS' + dazUnit+ '.PID'
    pid = getPid( filename )
    if pid :    
        # Killing background midas session
        #print 'Killing  PyMidas session  pid ',pid
        #killProcess( pid )
        # remove file 
            os.remove( filename )


    # remove Midas_osxxx
    filename =  os.path.join( midwork, 'Midas_osx'+ dazUnit )
    if os.path.exists(filename):
        os.remove( filename )



def killProcess( pid, _debug ):
    """ Kill process by give PID number"""
    status = pidStillRunning( pid )
    if status:
        try:
            if _debug:
                print 'killing process ', pid
            os.kill(pid, 9)
        except OSError:
            print 'Cannot kill process ',pid
            return 0

def getPid( filename ):
    """ Get PID number from given lock file"""
    if os.path.exists(filename):
        file = open( filename, 'r' )  #(2) Bug report : SOMO-6GLFK5
        try:
            pid = int( file.readline( ) )
            file.close( )
        except ValueError:
            print 'Warning! Cannot get PID number from file ', filename
            return 0
        return pid
    else:
        return 0

def getPidIDI( filename ):
    """ Get Midas session IDI PID number"""
    if os.path.exists(filename):
        file = open( filename, 'r' )
        file.readline( )
        pid = int( file.readline( ) )
        file.close( )
        return pid
    else:
        return 0


def pidStillRunning(  pid ):
    """ Check if process number PID is running"""
    try:
        os.kill(pid, 0)
    except OSError:
        print 'No process ',pid
        return 0
    return 1
