"""module midastask.py -- defines MidasTask  class

"""
##############################################################################
#   This file is part of PyMidas.
#
#   PyMidas is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   PyMidas is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with PyMidas; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
##############################################################################
#
#  Author:             S. Maisala       
#  Email:              pymidas@eso.org
#  Contact:            www.eso.org/sampo/pymidas
#
##############################################################################
#
#   Change History log
#
# Num  Date      Revision  Who   What
#
# 00   2005/07/15  0.95    SM   Creation of this History  
# 
# 01   2005/11/03  1.0.0   SM  Version 1.0.0
#
###############################################################################

import os, sys, string, types, copy, re
import pymidasCom ,midas

executionMonitor = None
_MidasTask_attr_dict = {
    '_command':None, 
    '_name': None,
    # parameters have a current set of values and a default set
    '_currentParList': None,
    '_hidden': 0,
    }


class MidasTask(midas.MidasTask):

    """MIDAS task class"""

    def __init__(self, name):
        # for this heavily used code, pull out the dictionary and
        # initialize it directly to avoid calls to __setattr__
        objdict = self.__dict__
        # stuff all the parameters into the object
        objdict.update(_MidasTask_attr_dict)
        sname = string.replace(name, '.', '_')
        #if sname != name:
            #print "Warning: '.' illegal in task name, changing", name, \
            #        "to", sname
        
        # tasks with names starting with '_' are implicitly hidden
        if name[0:1] == '_': objdict['_hidden'] = 1
        objdict['_name'] = sname
        
        

    def run(self,*args,**kw):
        
        command = self.getName()
        if executionMonitor:
                executionMonitor(self)
        for params in args:
            command= command +' '+ params 
        result = pymidasCom._run(command)
        if result !='':
            return result
    def __call__(self,*args,**kw):

        
        return apply(self.run,args,kw)


    def getName(self):      return self._name


#setattr(midas,'load_image', midas.midastask.MidasTask('load/image') )






            
